require_dependency "onedrive/application_controller"

module Onedrive
  class DriveController < ApplicationController

    require 'net/http'
    require 'uri'

    def onedrive_redirect
      Onedrive.mount_url + "/auth_redirect"
    end

    def onedrive_link
      permissions = "Files.ReadWrite.AppFolder Files.ReadWrite.All offline_access"
      @onedrive_link = "https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id=#{Onedrive.client_id}&scope=#{permissions}&response_type=code&redirect_uri=#{onedrive_redirect}"
    end

    def index
      session["redirect_url"] = params[:redirect_url]
      @onedrive_link = onedrive_link
      redirect_to @onedrive_link
    end

    def auth_complete
      code = params[:code]

      url = "https://login.microsoftonline.com/common/oauth2/v2.0/token"
      params = {
        :client_id => Onedrive.client_id,
        :redirect_uri => onedrive_redirect,
        :client_secret => Onedrive.client_secret,
        :code => code,
        :grant_type => "authorization_code"
      }

      uri = URI.parse(url)
      http = Net::HTTP.new(uri.host, uri.port)
      req = Net::HTTP::Post.new(uri.request_uri, 'Content-Type' => 'application/x-www-form-urlencoded')
      req.set_form_data(params)
      http.use_ssl = (uri.scheme == "https")
      response = http.request(req)

      if response.code[0] != '2'
        redirect_to "#{Onedrive.mount_url}/?secret_url=error"
      else
        user = Onedrive::User.new
        auth_params = JSON.parse(response.body)
        user.expires_at = DateTime.now + auth_params["expires_in"].to_i.seconds
        user.auth_params = auth_params.to_json
        user.token = SecureRandom.uuid
        user.save

        @secret_url = CGI.escape("#{Onedrive.mount_url}/sync?key=#{user.token}&type=sf")
        redirect_to "#{session["redirect_url"]}&onedrive_secret_url=#{@secret_url}"
      end
    end
  end
end
